﻿using System.ServiceModel.DomainServices.Client.ApplicationServices;
using System.Windows;
using System.Windows.Controls;
using RoomManagement.Views;

namespace RoomManagement
{
    public partial class App : Application
    {
        public App()
        {
            InitializeComponent();

            // Create a WebContext and add it to the ApplicationLifetimeObjects
            // collection. This will then be available as WebContext.Current.
            WebContext webContext = new WebContext();
            webContext.Authentication = new WindowsAuthentication();
            webContext.Authentication.LoadUser(OnUserLoaded, null);
            this.ApplicationLifetimeObjects.Add(webContext);
        }

        private void OnUserLoaded( LoadUserOperation luo )
        {
            // This will enable you to bind controls in XAML files to WebContext.Current properties
            this.Resources.Add("WebContext", WebContext.Current);
            if ( !WebContext.Current.User.IsReader )
            {
                Grid g = new Grid();
                TextBlock tb = new TextBlock { Text = Constants.NotAllowedReadingMessage };
                tb.HorizontalAlignment = HorizontalAlignment.Center;
                tb.VerticalAlignment = VerticalAlignment.Center;
                g.Children.Add( tb );
                this.RootVisual = g;
            }
            else
            {
                // Show some UI to the user while LoadUser is in progress (a.k.a. InitializeRootVisual)
                this.RootVisual = new MainPage();
            }
        }

        private void Application_UnhandledException( object sender, ApplicationUnhandledExceptionEventArgs e )
        {
            SilverMessageBox.Show( "An unhandled exception occurred. Please empty your cache and try again.\nIf this doesn't work, please contact the service desk ans include the following information about the error:", "OK", e.ExceptionObject.Message );
            e.Handled = true;
        }
    }
}