﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using RoomManagement.Views;
using RoomManagement.Web.Models;

namespace RoomManagement.Controls
{
    public partial class CellSelector : UserControl, INotifyPropertyChanged
    {
        public DependencyProperty SitesListProperty =
               DependencyProperty.Register( "SitesList", typeof( IEnumerable<Site> ), typeof( CellSelector ), null );

        public IEnumerable<Site> SitesList
        {
            get
            {
                return (IEnumerable<Site>) GetValue( SitesListProperty );
            }
            set
            {
                SetValue( SitesListProperty, value );
            }
        }

        public DependencyProperty RoomTypesListProperty =
               DependencyProperty.Register( "RoomTypesList", typeof( IEnumerable<RoomType> ), typeof( CellSelector ), null );

        public IEnumerable<RoomType> RoomTypesList
        {
            get
            {
                return (IEnumerable<RoomType>) GetValue( RoomTypesListProperty );
            }
            set
            {
                SetValue( RoomTypesListProperty, value );
            }
        }

        public DependencyProperty CellTypesListProperty =
               DependencyProperty.Register( "CellTypesList", typeof( IEnumerable<CellType> ), typeof( CellSelector ), null );

        public IEnumerable<CellType> CellTypesList
        {
            get
            {
                return (IEnumerable<CellType>) GetValue( CellTypesListProperty );
            }
            set
            {
                SetValue( CellTypesListProperty, value );
            }
        }

        public DependencyProperty CanChangeCellTypeProperty =
               DependencyProperty.Register( "CanChangeCellType", typeof( bool ), typeof( CellSelector ), null );

        public bool CanChangeCellType
        {
            get
            {
                return (bool) GetValue( CanChangeCellTypeProperty );
            }
            set
            {
                SetValue( CanChangeCellTypeProperty, value );
                FirePropertyChanged( "CanChangeCellType" );
            }
        }

        public Site SelectedSite
        {
            get
            {
                return (Site) DGsites.SelectedItem;
            }
        }
        public Floor SelectedFloor
        {
            get
            {
                return (Floor) DGfloors.SelectedItem;
            }
        }
        public Room SelectedRoom
        {
            get
            {
                return (Room) DGrooms.SelectedItem;
            }
        }
        public Cell SelectedCell
        {
            get
            {
                return (Cell) DGcells.SelectedItem;
            }
        }

        public CellSelector()
        {
            InitializeComponent();
            LayoutRoot.DataContext = this;
        }

        private void CBcellType_Loaded( object sender, RoutedEventArgs e )
        {
            ComboBox cb = (ComboBox) sender;

            if ( !CanChangeCellType )
            {
                cb.Visibility = Visibility.Collapsed;
            }
            else
            {
                cb.ItemsSource = CellTypesList;
                cb.SelectedItem = ( (Cell) cb.DataContext ).CellType;
            }

        }

        private void CBcellType_SelectionChanged( object sender, SelectionChangedEventArgs e )
        {
            ComboBox cb = (ComboBox) sender;
            Cell c = (Cell) cb.DataContext;
            CellType type = cb.SelectedItem as CellType;
            if ( type != null && c.People.Count > 1 && type.ID != Constants.DeskSharingCellTypeId )
            {
                SilverMessageBox.Show( "This cell has more than one person in it. It can only be a desk sharing place." );
                // removed, because the user no longer see where the problem is
                //cb.SelectedItem = CellTypesList.Where( ct => ct.ID == Constants.DeskSharingCTypeId ).First();
            }
        }

        #region INotifyPropertyChanged
        public event PropertyChangedEventHandler PropertyChanged;
        private void FirePropertyChanged( string propertyName )
        {
            if ( PropertyChanged != null )
            {
                PropertyChanged( this, new PropertyChangedEventArgs( propertyName ) );
            }
        }
        #endregion

        private void OnSiteSelected(object sender, SelectionChangedEventArgs e) {

            Site s = (Site) DGsites.SelectedItem;

            if (s != null && s.Floors.Count > 0) {

                DGfloors.SelectedIndex = 0;
            }
        }

        private void OnFloorSelected(object sender, SelectionChangedEventArgs e) {

            Floor f = (Floor) DGfloors.SelectedItem;

            if (f != null && f.Rooms.Count > 0) {
                DGrooms.SelectedIndex = 0;
            }
        }

        private void OnRoomSelected(object sender, SelectionChangedEventArgs e) {

            Room r = (Room) DGrooms.SelectedItem;

            if (r != null && r.Cells.Count > 0) {

                DGcells.SelectedIndex = 0;
            }
        }
    }
}