﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using RoomManagement.Web.Models;

namespace RoomManagement.Controls
{
    public partial class SitesSelector : UserControl
    {
        public DependencyProperty SitesListProperty = DependencyProperty.Register( "SitesList", typeof( IEnumerable<Site> ), typeof( SitesSelector ), null );
        public IEnumerable<Site> SitesList
        {
            get
            {
                return (IEnumerable<Site>) GetValue( SitesListProperty );
            }
            set
            {
                SetValue( SitesListProperty, value );
            }
        }

        public List<Site> SelectedSites;

        public event EventHandler SelectedSitesChanged;

        public SitesSelector()
        {
            InitializeComponent();
            LayoutRoot.DataContext = this;
            SelectedSites = new List<Site>();
        }

        public void ClearText()
        {
            // Used to clear text in PeopleManagement
            TXTselected.Text = string.Empty;
        }

        private void ComboBox_DropDownOpened( object sender, EventArgs e )
        {
            SetText();
        }
        private void ComboBox_DropDownClosed( object sender, EventArgs e )
        {
            CBsites.SelectedItem = null;
            SetText();
        }

        private void SiteCheckBox_Checked( object sender, RoutedEventArgs e )
        {
            CheckBox cb = (CheckBox) sender;
            Site selectedSite = (Site) cb.DataContext;
            SelectedSites.Add( selectedSite );
            FireSelectedSitesChanged();
            SetText();
        }
        private void SiteCheckBox_Unchecked( object sender, RoutedEventArgs e )
        {
            CheckBox cb = (CheckBox) sender;
            Site selectedSite = (Site) cb.DataContext;
            SelectedSites.Remove( selectedSite );
            FireSelectedSitesChanged();
            SetText();
        }

        private void FireSelectedSitesChanged()
        {
            if (SelectedSitesChanged != null)
            {
                SelectedSitesChanged(this, EventArgs.Empty);
            }

        }

        private void SetText()
        {
            if ( SelectedSites.Count > 0 )
            {
                SelectedSites.Sort( ( a, b ) => a.Name.CompareTo( b.Name ) );
                string names = string.Empty;
                foreach ( Site s in SelectedSites )
                {
                    names += s.Name + ", ";
                }
                names = names.Remove( names.Length - 2 ); // remove the last ", "
                if ( names.Length > 25 ) // max chars before it gets out of the combobox
                {
                    names = names.Remove( 23 );
                    names += " ...";
                }
                TXTselected.Text = names;
            }
            else
            {
                TXTselected.Text = string.Empty;
            }
        }
    }
}