﻿using System;
using System.ComponentModel;
using System.Linq;
using System.ServiceModel.DomainServices.Client;
using System.Windows;
using System.Windows.Data;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using RoomManagement.Web.Models;

namespace RoomManagement
{
    public class NotOperatorValueConverter : IValueConverter
    {
        public object Convert( object value, Type targetType, object parameter, System.Globalization.CultureInfo culture )
        {
            return !( (bool) value );
        }

        public object ConvertBack( object value, Type targetType, object parameter, System.Globalization.CultureInfo culture )
        {
            return !( (bool) value );
        }
    }
    public class FloorSortConverter : IValueConverter
    {
        public object Convert( object value, Type targetType, object parameter, System.Globalization.CultureInfo culture )
        {
            return ( (EntityCollection<Floor>) value ).OrderBy( f => f.Number );
        }

        public object ConvertBack( object value, Type targetType, object parameter, System.Globalization.CultureInfo culture )
        {
            throw new NotSupportedException( "If you see this message, something very wrong happened." );
        }
    }
    public class RoomsSortConverter : IValueConverter
    {
        public object Convert( object value, Type targetType, object parameter, System.Globalization.CultureInfo culture )
        {
            return ( (EntityCollection<Room>) value ).OrderBy( r => r.Number );
        }

        public object ConvertBack( object value, Type targetType, object parameter, System.Globalization.CultureInfo culture )
        {
            throw new NotSupportedException( "If you see this message, something very wrong happened." );
        }
    }
    public class CellsSortConverter : IValueConverter
    {
        public object Convert( object value, Type targetType, object parameter, System.Globalization.CultureInfo culture )
        {
            return ( (EntityCollection<Cell>) value ).OrderBy( c => c.Number );
        }

        public object ConvertBack( object value, Type targetType, object parameter, System.Globalization.CultureInfo culture )
        {
            throw new NotSupportedException( "If you see this message, something very wrong happened." );
        }
    }
    public class BoolToFontWeightConverter : IValueConverter
    {
        public object Convert( object value, Type targetType, object parameter, System.Globalization.CultureInfo culture )
        {
            bool b = (bool) value;
            return b ? FontWeights.ExtraBold : FontWeights.Normal;
        }

        public object ConvertBack( object value, Type targetType, object parameter, System.Globalization.CultureInfo culture )
        {
            throw new NotSupportedException( "If you see this message, something very wrong happened." );
        }
    }
    public class BoolToFontStyleConverter : IValueConverter
    {
        public object Convert( object value, Type targetType, object parameter, System.Globalization.CultureInfo culture )
        {
            bool b = (bool) value;
            return b ? FontStyles.Normal : FontStyles.Italic;
        }

        public object ConvertBack( object value, Type targetType, object parameter, System.Globalization.CultureInfo culture )
        {
            throw new NotSupportedException( "If you see this message, something very wrong happened." );
        }
    }
    public class EmptyWorkPlaceConverter : IValueConverter
    {
        public object Convert( object value, Type targetType, object parameter, System.Globalization.CultureInfo culture )
        {
            string s = (string) value;
            return ( string.IsNullOrWhiteSpace( s ) ? "nowhere" : s );
        }

        public object ConvertBack( object value, Type targetType, object parameter, System.Globalization.CultureInfo culture )
        {
            throw new NotSupportedException( "If you see this message, something very wrong happened." );
        }
    }
    public class EmptyIntConverter : IValueConverter
    {
        public object Convert( object value, Type targetType, object parameter, System.Globalization.CultureInfo culture )
        {
            int n = (int) value;
            return ( n > 0 ? n.ToString() : "none" );
        }

        public object ConvertBack( object value, Type targetType, object parameter, System.Globalization.CultureInfo culture )
        {
            throw new NotSupportedException( "If you see this message, something very wrong happened." );
        }
    }
    public class BoolToVisibilityConverter : IValueConverter
    {
        public object Convert( object value, Type targetType, object parameter, System.Globalization.CultureInfo culture )
        {
            return ( (bool) value ) ? Visibility.Visible : Visibility.Collapsed;
        }

        public object ConvertBack( object value, Type targetType, object parameter, System.Globalization.CultureInfo culture )
        {
            throw new NotSupportedException( "If you see this message, something very wrong happened." );
        }
    }

    public class HypothesisChangeTypeToImagePathConverter : IValueConverter
    {
        public object Convert( object value, Type targetType, object parameter, System.Globalization.CultureInfo culture ) {

            HypothesisChangeType ct = (HypothesisChangeType) value;

            //NOTE: unable to use absolute formats to point to the image resource...
            string url;
            switch (ct) {
                case HypothesisChangeType.Added:
                    url = "../Images/Add.png";
                    break;

                case HypothesisChangeType.Deleted:
                    url = "../Images/Del.png";
                    break;

                case HypothesisChangeType.Modified:
                    url = "../Images/Modified.png";
                    break;

                //case HypothesisChangeType.Unchanged:
                default:
                    return null;
            }
            return url;
        }

        public object ConvertBack( object value, Type targetType, object parameter, System.Globalization.CultureInfo culture )
        {
            throw new NotSupportedException( "If you see this message, something very wrong happened." );
        }
    }
}