﻿using System;
using System.Collections;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

namespace RoomManagement {

    /// <summary>
    /// Miscellaneous helper methods
    /// </summary>
    public static class DataGridHelper {

        private static FrameworkElement GetParent(FrameworkElement child, Type targetType) {
            object parent = child.Parent;
            if (parent != null) {
                if (parent.GetType() == targetType) {
                    return (FrameworkElement)parent;
                }
                return GetParent((FrameworkElement)parent, targetType);
            }
            return null;
        }

        /// <summary>
        /// Sets the <c>ComboBox.ItemsSource</c> property of a <c>ComboBox</c> somewhere in a <c>DataGrid</c>
        /// </summary>
        /// <param name="dg">The <c>DataGrid</c> with a <c>ComboBox</c> in a column</param>
        /// <param name="row">The involved row in the <c>DataGrid</c></param>
        /// <param name="source">The items list of the <c>ComboBox</c></param>
        public static void SetItemsSource(DataGrid dg, DataGridRow row, IEnumerable source) {

            foreach (DataGridColumn dgc in dg.Columns) {

                FrameworkElement fe = dgc.GetCellContent(row);
                FrameworkElement parent = GetParent(fe, typeof (DataGridCell));

                if (parent != null) {
                    DataGridCell celldata = (DataGridCell) parent;
                    ComboBox cb = celldata.Content as ComboBox;
                    if (cb != null) {
                        cb.ItemsSource = source;
                        return; // only one combo box
                    }
                }

            }
        }

        public static DataGridColumn FindColumnBySortPath(this DataGrid dg, string sortPath) {

            return dg.Columns.FirstOrDefault(dgc => dgc.SortMemberPath == sortPath);
        }
    }
}
