﻿using System;
using System.ComponentModel;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Navigation;
using RoomManagement.Views;
using RoomManagement.Web.Models;

namespace RoomManagement
{
    public partial class MainPage : UserControl, INotifyPropertyChanged
    {
        private ICanBeBusy _busyPage;
        private IWantFocusNotification _previousPage;

        public bool IsBusy
        {
            get
            {
                if ( _busyPage == null ) return false;
                return _busyPage.IsBusy;
            }
        }

        public bool CanSeeExperimentalFeatures {
            get {
                return WebContext.Current.User.Name == @"ELCA\jmp";
            }
        }

        public MainPage()
        {
            InitializeComponent();
            DataContext = this;
            if ( !WebContext.Current.User.IsAdministrator )
            {
                HLrooms.Visibility = Visibility.Collapsed;
            }
        }

        private void RefreshButton_Click( object sender, RoutedEventArgs e )
        {
            Page p = (Page) ContentFrame.Content;
            IRefreshable r = (IRefreshable) p;
            r.RefreshData();
        }

        // After the Frame navigates, ensure the HyperlinkButton representing the current page is selected
        private void ContentFrame_Navigated( object sender, NavigationEventArgs e )
        {
            foreach ( UIElement child in LinksStackPanel.Children )
            {
                HyperlinkButton hb = child as HyperlinkButton;
                if ( hb != null && hb.NavigateUri != null )
                {
                    VisualStateManager.GoToState(hb, (hb.NavigateUri.ToString().Equals(e.Uri.ToString()) ? "ActiveLink" : "InactiveLink"), true);
                }
            }

            if (_previousPage != null) {
                _previousPage.HasLostFocus();
            }

            Page p = (Page) ContentFrame.Content;
            RefreshButton.IsEnabled = ( ( p as IRefreshable ) != null );
            if ( _busyPage != null )
            {
                _busyPage.IsBusyChanged -= OnBusyChanged;
            }
            _busyPage = p as ICanBeBusy;
            if ( _busyPage != null )
            {
                _busyPage.IsBusyChanged += OnBusyChanged;
            }

            // now, this is more like the current page...
            _previousPage = p as IWantFocusNotification;
            if (_previousPage != null) {
                _previousPage.HasGotFocus();
            }
        }

        private void OnBusyChanged( object sender, EventArgs e )
        {
            FirePropertyChanged( "IsBusy" );
        }

        private void ContentFrame_NavigationFailed( object sender, NavigationFailedEventArgs e )
        {
            e.Handled = true;
            SilverMessageBox.Show( e.Exception.Message, "OK", e.Exception.ToString() );
        }

        #region INotifyPropertyChanged
        public event PropertyChangedEventHandler PropertyChanged;
        private void FirePropertyChanged( string propertyName )
        {
            if ( PropertyChanged != null )
            {
                PropertyChanged( this, new PropertyChangedEventArgs( propertyName ) );
            }
        }
        #endregion
    }
}