﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Globalization;
using System.Linq;
using System.Net;
using System.ServiceModel.DomainServices.Client;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Data;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Shapes;
using System.Windows.Navigation;
using RoomManagement.Web.Models;
using RoomManagement.Web.Services;

namespace RoomManagement.Views {
    public partial class Floors : /* Page, */ INotifyPropertyChanged {

        // the following IDs have to be reset if the DB is recreated
        private const int LausanneFlon2 = 1;
        private const int LausanneHarpe1 = 2;
        private const int LausanneHarpe2 = 3;
        private const int LausanneHarpe3 = 4;
        private const int LausanneHarpe4 = 5;
        private const int LausanneHarpe5 = 6;
        private const int LausanneHarpe6 = 7;
        private const int Geneva5 = 8;
        private const int Berne2 = 9;
        private const int Berne3 = 10;
        private const int ZurichStein2 = 12;
        private const int ZurichStein4 = 14;
        private const int ZurichBirmendorf1 = 15;
        private const int Neuchatel1 = 16;
        private const int Madrid8 = 17;
        private const int Paris = 18;

        private IChangeChildren _lastFloorControl;
        private readonly RoomContext _rc = new RoomContext();
        private bool _isGraphicsChangeUIVisible;

        public Floors() {
            InitializeComponent();

            _rc.Load(_rc.GetFloorsWithGraphicsInformationQuery());

            _isGraphicsChangeUIVisible = true;
            DataContext = this;
        }

        public EntitySet<Floor> AllFloors {
            get { return _rc.Floors; }
        }

        public bool IsGraphicsChangeUIVisible {
            get { return _isGraphicsChangeUIVisible; }
            set {
                if (_isGraphicsChangeUIVisible != value) {
                    _isGraphicsChangeUIVisible = value;
                    FirePropertyChanged("IsGraphicsChangeUIVisible");
                }
            }
        }
        // Executes when the user navigates to this page.
        protected override void OnNavigatedTo(NavigationEventArgs e) {
        }

        private void OnFloorChanged(object sender, SelectionChangedEventArgs e) {
            Floor f = (Floor) FloorsList.SelectedItem;
            if (f != null) {
                DrawFloor(f);
            }
        }

        private void DrawFloor(Floor f) {

            IChangeChildren floorControl = null;

            switch (f.ID) {
                case LausanneHarpe5:
                    floorControl = new LausanneHarpe5Floor();
                    break;
            }

            if (_lastFloorControl != null) {
                LayoutRoot.Children.Remove(_lastFloorControl.Root);
            }

            if (floorControl == null) {
                floorControl = new CustomTextBlock("Sorry, this floor is not yet known in the software. Talk to its responsible person");
            }

            _lastFloorControl = floorControl;

            DrawRooms(f);
            LayoutRoot.Children.Add(_lastFloorControl.Root);
            Grid.SetRow(_lastFloorControl.Root, 1);
        }

        private void DrawRooms(Floor f) {
            
            foreach (Room r in f.Rooms) {
                DrawRoom(r);
            }
        }

        private void DrawRoom(Room r) {

            Canvas c = new Canvas {
                    Name = GetRoomId(r)
            };
            if (r.XCoordinateInFloor.HasValue && r.YCoordinateInFloor.HasValue) {
                Position(c, r.XCoordinateInFloor.Value, r.YCoordinateInFloor.Value);
            }

            RoomLabel rl = new RoomLabel(r.Number.ToString());
            if (r.NumberXCoordinate.HasValue && r.NumberYCoordinate.HasValue) {
                Position(rl, r.NumberXCoordinate.Value, r.NumberYCoordinate.Value);
            }
            c.Children.Add(rl);

            DrawWalls(r.RoomPoints, c);
            DrawDesks(r.Cells, c);

            _lastFloorControl.Add(c);
        }


        private void RedrawRoom(Room r) {

            string roomId = GetRoomId(r);
            _lastFloorControl.RemoveById(roomId);

            DrawRoom(r);
        }

        private static string GetRoomId(Room r) {
            return "r" + r.ID;
        }

        private static void DrawWalls(IEnumerable<RoomPoint> points, Canvas parent) {

#if !POLYLINE
            Brush simpleBrush = new SolidColorBrush(Colors.Gray);
            Brush thinBrush = new SolidColorBrush(Colors.LightGray);

            // these 2 initializations are forced upon us by the compiler, but are useless
            Point doorStartPoint = new Point();
            Point doorMinPoint = new Point(); // the top-left point of the whole door
            Polyline p = null;

            foreach (RoomPoint rp in points) {

                PointType pt = (PointType) rp.PointType;
                Point currentPoint = new Point(rp.XCoordinate, rp.YCoordinate);

                if ((pt & PointType.WallStart) != 0) {
                    p = new Polyline {
                        //Fill = new SolidColorBrush(Colors.Gray),
                        Stroke = simpleBrush,
                        StrokeThickness = 3,

                    };
                    p.Points.Add(currentPoint);
                }

                if ((pt & PointType.WallJunction) != 0) {

                    if (p == null) {
                        throw new InvalidOperationException("A wall junction should be preceded by a wall start");
                    }
                    p.Points.Add(currentPoint);
                }

                if ((pt & PointType.WallEnd) != 0) {
                    if (p == null) {
                        throw new InvalidOperationException("A wall end should be preceded by a wall start");
                    }
                    p.Points.Add(currentPoint);
                    parent.Children.Add(p);
                    p = null;
                }

                if ((pt & PointType.MovableWallStart) != 0) {
                    p = new Polyline {
                        Stroke = thinBrush,
                        StrokeThickness = 2,
                    };
                    p.Points.Add(currentPoint);
                }

                if ((pt & PointType.MovableWallJunction) != 0) {
                    if (p == null) {
                        throw new InvalidOperationException("A movable wall junction should be preceded by a movable wall start");
                    }
                    p.Points.Add(currentPoint);
                }

                if ((pt & PointType.MovableWallEnd) != 0) {
                    if (p == null) {
                        throw new InvalidOperationException("A movable wall end should be preceded by a movable wall start");
                    }
                    p.Points.Add(currentPoint);
                    parent.Children.Add(p);
                    p = null;
                }

                if ((pt & PointType.DoorPivot) != 0) {
                    doorStartPoint = currentPoint;
                    doorMinPoint = currentPoint;
                }

                if ((pt & PointType.DoorOpened) != 0) {

                    Line l = new Line
                    {
                        Stroke = simpleBrush,
                        StrokeThickness = 1,
                        X1 = doorStartPoint.X,
                        Y1 = doorStartPoint.Y,
                        X2 = currentPoint.X,
                        Y2 = currentPoint.Y,
                    };
                    parent.Children.Add(l);
                    doorMinPoint.X = Math.Min(doorMinPoint.X, currentPoint.X);
                    doorMinPoint.Y = Math.Min(doorMinPoint.Y, currentPoint.Y);
                }

                if ((pt & PointType.DoorClosed) != 0) {

                    doorMinPoint.X = Math.Min(doorMinPoint.X, currentPoint.X);
                    doorMinPoint.Y = Math.Min(doorMinPoint.Y, currentPoint.Y);
                    
                    Line l = new Line
                    {
                        Stroke = simpleBrush,
                        StrokeThickness = 1,
                        StrokeDashArray = new DoubleCollection { 2, 2 },
                        X1 = doorStartPoint.X,
                        Y1 = doorStartPoint.Y,
                        X2 = currentPoint.X,
                        Y2 = currentPoint.Y,
                    };
                    parent.Children.Add(l);

                    double doorWidth =
                        Math.Sqrt(Math.Pow(doorStartPoint.X - currentPoint.X, 2) +
                                  Math.Pow(doorStartPoint.Y - currentPoint.Y, 2));

                    Point ellipseTopLeft = new Point(doorStartPoint.X - doorWidth, doorStartPoint.Y - doorWidth);
                    
                    Ellipse el = new Ellipse
                    {
                        Stroke = simpleBrush,
                        StrokeThickness = 1,
                        Height = 2 * doorWidth,
                        Width = 2 * doorWidth,
                        StrokeDashArray = new DoubleCollection { 2, 2 }, // cannot be shared...
                        Clip = new RectangleGeometry
                        {
                            Rect = new Rect(
                                doorMinPoint.X - ellipseTopLeft.X, doorMinPoint.Y - ellipseTopLeft.Y, 
                                doorWidth, doorWidth)
                        },
                    };
                    Canvas.SetTop(el, ellipseTopLeft.Y);
                    Canvas.SetLeft(el, ellipseTopLeft.X);
                    parent.Children.Add(el);
                }
            }
#else
/* expected result:
 <Path Stroke="Black" StrokeThickness="1" >
    <Path.Data>
      <PathGeometry>
        <PathGeometry.Figures>
          <PathFigure StartPoint="10,50">
            <PathFigure.Segments>
              <BezierSegment Point1="100,0"
                Point2="200,200"
                Point3="300,100"/>
              <LineSegment Point="400,100" />
              <ArcSegment Size="50,50" RotationAngle="45"
                IsLargeArc="True" SweepDirection="Clockwise"
                Point="200,100"/>
            </PathFigure.Segments>
          </PathFigure>
        </PathGeometry.Figures>
      </PathGeometry>
    </Path.Data>
  </Path>
 */
            Path p = new Path();
            PathGeometry pg = new PathGeometry();
            PathFigureCollection pfc = new PathFigureCollection();
            PathFigure pf = null;

            foreach (RoomPoint rp in points) {

                PointType pt = (PointType) rp.PointType;
                Point currentPoint = new Point(rp.XCoordinate, rp.YCoordinate);

                if ((pt & PointType.WallStart) != 0) {

                    if (pf != null) {
                        pfc.Add(pf);
                    }
                    pf = new PathFigure  { 
                        StartPoint = currentPoint
                    };
                    p = new Polyline {
                        //Fill = new SolidColorBrush(Colors.Gray),
                        Stroke = simpleBrush,
                        StrokeThickness = 3,

                    };
                    p.Points.Add(currentPoint);
                }

                if ((pt & PointType.WallJunction) != 0) {

                    if (p == null) {
                        throw new InvalidOperationException("A wall junction should be preceded by a wall start");
                    }
                    p.Points.Add(currentPoint);
                }

                if ((pt & PointType.WallEnd) != 0) {
                    if (p == null) {
                        throw new InvalidOperationException("A wall end should be preceded by a wall start");
                    }
                    p.Points.Add(currentPoint);
                    parent.Children.Add(p);
                    p = null;
                }

                if ((pt & PointType.MovableWallStart) != 0) {
                    p = new Polyline {
                        Stroke = thinBrush,
                        StrokeThickness = 2,
                    };
                    p.Points.Add(currentPoint);
                }

                if ((pt & PointType.MovableWallJunction) != 0) {
                    if (p == null) {
                        throw new InvalidOperationException("A movable wall junction should be preceded by a movable wall start");
                    }
                    p.Points.Add(currentPoint);
                }

                if ((pt & PointType.MovableWallEnd) != 0) {
                    if (p == null) {
                        throw new InvalidOperationException("A movable wall end should be preceded by a movable wall start");
                    }
                    p.Points.Add(currentPoint);
                    parent.Children.Add(p);
                    p = null;
                }

                if ((pt & PointType.DoorPivot) != 0) {
                    doorStartPoint = currentPoint;
                }

                if ((pt & PointType.DoorOpened) != 0) {

                    Line l = new Line
                    {
                        Stroke = simpleBrush,
                        StrokeThickness = 1,
                        X1 = doorStartPoint.X,
                        Y1 = doorStartPoint.Y,
                        X2 = rp.XCoordinate,
                        Y2 = rp.YCoordinate,
                    };
                    parent.Children.Add(l);

                }

                if ((pt & PointType.DoorClosed) != 0) {
                    Line l = new Line
                    {
                        Stroke = simpleBrush,
                        StrokeThickness = 1,
                        StrokeDashArray = new DoubleCollection { 2, 2 },
                        X1 = doorStartPoint.X,
                        Y1 = doorStartPoint.Y,
                        X2 = rp.XCoordinate,
                        Y2 = rp.YCoordinate,
                    };
                    parent.Children.Add(l);

                    double doorWidth =
                        Math.Sqrt(Math.Pow(doorStartPoint.X - rp.XCoordinate, 2) +
                                  Math.Pow(doorStartPoint.Y - rp.YCoordinate, 2));
                    // todo: manage angles
                    Ellipse el = new Ellipse
                    {
                        Stroke = simpleBrush,
                        StrokeThickness = 1,
                        Height = 2 * doorWidth,
                        Width = 2 * doorWidth,
                        StrokeDashArray = new DoubleCollection { 2, 2 }, // cannot be shared...
                        Clip = new RectangleGeometry
                        {
                            Rect = new Rect(doorWidth, 0, 2 * doorWidth, doorWidth)
                        },
                    };
                    Canvas.SetTop(el, doorStartPoint.Y - doorWidth);
                    Canvas.SetLeft(el, doorStartPoint.X - doorWidth);
                    parent.Children.Add(el);
                }
            }

            pfc.Add(pf);
            pg.Figures = pfc;
            p.Data = pg;
            parent.Children.Add(p);
#endif

        }

        private static void DrawDesks(IEnumerable<Cell> cells, Canvas parent) {

            foreach (Cell c in cells) {

                Desk d = new Desk(c);
                parent.Children.Add(d);
            }
        }

        private static void Position(UIElement elt, double xCoordinate, double yCoordinate) {

            Canvas.SetTop(elt, yCoordinate);
            Canvas.SetLeft(elt, xCoordinate);
        }

        #region INotifyPropertyChanged
        public event PropertyChangedEventHandler PropertyChanged;
        private void FirePropertyChanged(string propertyName) {
            if (PropertyChanged != null) {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }
        #endregion

        private void OnAddRoomPoint(object sender, RoutedEventArgs e) {
            Room r = (Room) _lbRooms.SelectedItem;

            if (r != null) {
                RoomPoint rp = new RoomPoint();
                r.RoomPoints.Add(rp);
            }
        }

        private void OnDeleteRoomPoint(object sender, RoutedEventArgs e) {
            RoomPoint rp = (RoomPoint) _dgPoints.SelectedItem;
            if (rp != null) {
                _rc.RoomPoints.Remove(rp);
            }
        }

        //private void OnLoadingRoomPointRow(object sender, DataGridRowEventArgs e) {

        //    DataGridColumn dgc = _dgPoints.Columns[0]; // the combo box is known to be at column zero
        //    FrameworkElement fe = dgc.GetCellContent(e.Row);

        //    ComboBox cb = fe as ComboBox;
        //    if (cb != null) { 
        //        cb.ItemsSource = PointTypes;
        //    }
        //}

        private void OnUpdateGraphics(object sender, RoutedEventArgs e) {
            Room r = (Room) _lbRooms.SelectedItem;
            RedrawRoom(r);
        }

        private void OnSaveChanges(object sender, RoutedEventArgs e) {
            _rc.SubmitChanges();
        }

        private void OnShowAddOptions(object sender, RoutedEventArgs e) {
            ShowAddOptions();
        }

        private void ShowAddOptions() {
            _addMenu.IsOpen = true;
        }

        private void AddDoor(Room r) {

            RoomPoint rp = new RoomPoint {PointType = (int) PointType.DoorPivot};
            if (r.RoomPoints.Count > 0) {
                RoomPoint lastPoint = r.RoomPoints.Last();
                rp.XCoordinate = lastPoint.XCoordinate;
                rp.YCoordinate = lastPoint.YCoordinate;
            }
            r.RoomPoints.Add(rp);

            RoomPoint rp2 = new RoomPoint {
                PointType = (int) PointType.DoorOpened,
                XCoordinate = rp.XCoordinate, 
                YCoordinate = rp.YCoordinate - 25
            };
            r.RoomPoints.Add(rp2);

            RoomPoint rp3 = new RoomPoint {
                PointType = (int) PointType.DoorOpened,
                XCoordinate = rp.XCoordinate + 25, 
                YCoordinate = rp.YCoordinate
            };
            r.RoomPoints.Add(rp3);
        }

        private void Add2Wall(Room r) {

            RoomPoint rp = new RoomPoint {PointType = (int) PointType.WallStart};
            if (r.RoomPoints.Count > 0) {
                RoomPoint lastPoint = r.RoomPoints.Last();
                rp.XCoordinate = lastPoint.XCoordinate;
                rp.YCoordinate = lastPoint.YCoordinate;
            }
            r.RoomPoints.Add(rp);
            RoomPoint rp2 = new RoomPoint {PointType = (int) PointType.WallEnd};
            r.RoomPoints.Add(rp2);
        }

        private void Add3Wall(Room r) {

            RoomPoint rp = new RoomPoint {PointType = (int) PointType.WallStart};
            if (r.RoomPoints.Count > 0) {
                RoomPoint lastPoint = r.RoomPoints.Last();
                rp.XCoordinate = lastPoint.XCoordinate;
                rp.YCoordinate = lastPoint.YCoordinate;
            }
            r.RoomPoints.Add(rp);
            RoomPoint rp2 = new RoomPoint {PointType = (int) PointType.WallJunction};
            r.RoomPoints.Add(rp2);
            RoomPoint rp3 = new RoomPoint {PointType = (int) PointType.WallEnd};
            r.RoomPoints.Add(rp3);
       }

        private void OnAddShortcutSelected(object sender, SelectionChangedEventArgs e) {

            int index = _lbxShortcuts.SelectedIndex;

            if (index != -1) {

                Room r = (Room) _lbRooms.SelectedItem;

                if (r != null) {

                    switch (index) {

                        case 0:
                            AddDoor(r);
                            break;

                        case 1:
                            Add2Wall(r);
                            break;

                        case 2:
                            Add3Wall(r);
                            break;
                    }
                }
            }

            _addMenu.IsOpen = false;
        }
    }

    public class RoomLabel : ContentControl {
        
        public RoomLabel(string text) {

            Border b = new Border {
                BorderBrush = new SolidColorBrush(Colors.Blue), 
                BorderThickness = new Thickness(2),
                CornerRadius = new CornerRadius(5),
            };
            TextBlock tb = new TextBlock {
                Text = text,
                Foreground = b.BorderBrush,
                FontSize = 14,
                Margin = new Thickness(2),
            };
            b.Child = tb;
            Content = b;
        }
    }

    public class Desk : ContentControl {

        public Desk(Cell c) {

            Brush usedSeatBrush = new SolidColorBrush(Colors.LightGray);
            Brush freeSeatBrush = new SolidColorBrush(new Color { A = 0xFF, R = 0xBA, G = 0xDB, B = 0x7D });
            Brush phoneBrush = new SolidColorBrush(new Color { A = 0xFF, R = 0x1E, G = 0x8E, B = 0x61 });
            Brush visaBrush = new SolidColorBrush(new Color { A = 0xFF, R = 0xDC, G = 0x6C, B = 0x7E });
            Brush outlineBrush = new SolidColorBrush(Colors.Black);

            Canvas cv = new Canvas();
            if (c.XCoordinate.HasValue && c.YCoordinate.HasValue) {
                Canvas.SetLeft(this, c.XCoordinate.Value);
                Canvas.SetTop(this, c.YCoordinate.Value);
            }

            Person p = c.People.FirstOrDefault();
            bool hasCellPhone = p != null && !String.IsNullOrEmpty(p.PhoneNumber);
            bool isCellOccupied = p != null && !String.IsNullOrEmpty(p.Visa);

            if (c.Orientation == (byte) SeatOrientation.East || c.Orientation == (byte) SeatOrientation.West) {

                // elements are positioned from left to right or right to left

                // seat
                Brush seatBrush = isCellOccupied ? usedSeatBrush : freeSeatBrush;
                Rectangle frontSeat = new Rectangle
                {
                    Height = 15,
                    Width = 10,
                    Stroke = outlineBrush,
                    StrokeThickness = 0.5,
                    Fill = seatBrush,
                };
                Ellipse backSeat = new Ellipse
                {
                    Fill = seatBrush,
                    Width = 6,
                    Height = 15,
                    Stroke = outlineBrush,
                    StrokeThickness = 0.5,
                };

                // desk itself
                Rectangle desk = new Rectangle
                {
                    Width = 27,
                    Height = 59,
                    Fill = seatBrush,
                    Stroke = outlineBrush,
                    StrokeThickness = 0.5,
                };

                if (c.Orientation == (byte) SeatOrientation.West) {
                    Canvas.SetLeft(backSeat, 0);
                    Canvas.SetTop(backSeat, 23);
                    cv.Children.Add(backSeat);

                    Canvas.SetLeft(frontSeat, 3);
                    Canvas.SetTop(frontSeat, 23);
                    cv.Children.Add(frontSeat);

                    Canvas.SetLeft(desk, 16);
                    Canvas.SetTop(desk, 0);
                    cv.Children.Add(desk);
                } else {
                    // orientation == East

                    Canvas.SetLeft(desk, 0);
                    Canvas.SetTop(desk, 0);
                    cv.Children.Add(desk);

                    Canvas.SetLeft(backSeat, 37);
                    Canvas.SetTop(backSeat, 23);
                    cv.Children.Add(backSeat);

                    Canvas.SetLeft(frontSeat, 30);
                    Canvas.SetTop(frontSeat, 23);
                    cv.Children.Add(frontSeat);
                }

                if (isCellOccupied) {

                    Border visa = new Border
                    {
                        CornerRadius = new CornerRadius(3),
                        BorderBrush = visaBrush,
                        BorderThickness = new Thickness(0.8),
                    };
                    Canvas.SetLeft(visa, Canvas.GetLeft(desk) + 4);
                    Canvas.SetTop(visa, 2);
                    TextBlock tbVisa = new TextBlock
                    {
                        Text = p.Visa,
                        Foreground = visaBrush,
                        FontSize = 8,
                        Margin = new Thickness(1, 0, 1, 0),
                    };
                    visa.Child = tbVisa;
                    cv.Children.Add(visa);
                }

                if (hasCellPhone) {

                    Border phone = new Border
                    {
                        CornerRadius = new CornerRadius(3),
                        BorderBrush = phoneBrush,
                        BorderThickness = new Thickness(0.8),
                    };
                    Canvas.SetLeft(phone, Canvas.GetLeft(desk) + 4);
                    Canvas.SetTop(phone, 22);
                    TextBlock tbPhone = new TextBlock
                    {
                        Text = p.PhoneNumber,
                        Foreground = phoneBrush,
                        FontSize = 8,
                        Margin = new Thickness(1, 0, 1, 0),
                    };
                    phone.Child = tbPhone;
                    cv.Children.Add(phone);
                }


                Border number = new Border
                {
                    CornerRadius = new CornerRadius(10),
                    BorderBrush = outlineBrush,
                    BorderThickness = new Thickness(0.8),
                };
                Canvas.SetLeft(number, Canvas.GetLeft(desk) + 7);
                Canvas.SetTop(number, 42);
                TextBlock tbnumber = new TextBlock
                {
                    Text = c.Number.ToString(),
                    FontSize = 8,
                    Margin = new Thickness(3, 0, 2, 0),
                };
                number.Child = tbnumber;
                cv.Children.Add(number);
            } else {
                // orientation north or South, so elements are positioned from top to to bottom or bottom to top
                // Note: this is essentially the same code as above, with width and height reversed
                // coordinates are also computed differently, so having one piece of code only would be very messy

                // seat
                Brush seatBrush = isCellOccupied ? usedSeatBrush : freeSeatBrush;
                Rectangle frontSeat = new Rectangle
                {
                    Height = 10,
                    Width = 15,
                    Stroke = outlineBrush,
                    StrokeThickness = 0.5,
                    Fill = seatBrush,
                };
                Ellipse backSeat = new Ellipse
                {
                    Fill = seatBrush,
                    Width = 15,
                    Height = 6,
                    Stroke = outlineBrush,
                    StrokeThickness = 0.5,
                };

                // desk itself
                Rectangle desk = new Rectangle
                {
                    Width = 59,
                    Height = 27,
                    Fill = seatBrush,
                    Stroke = outlineBrush,
                    StrokeThickness = 0.5,
                };

                if (c.Orientation == (byte) SeatOrientation.North) {

                    Canvas.SetLeft(backSeat, 23);
                    Canvas.SetTop(backSeat, 0);
                    cv.Children.Add(backSeat);

                    Canvas.SetLeft(frontSeat, 23);
                    Canvas.SetTop(frontSeat, 3);
                    cv.Children.Add(frontSeat);

                    Canvas.SetLeft(desk, 0);
                    Canvas.SetTop(desk, 16);
                    cv.Children.Add(desk);

                    Border number = new Border
                    {
                        CornerRadius = new CornerRadius(10),
                        BorderBrush = outlineBrush,
                        BorderThickness = new Thickness(0.8),
                    };
                    Canvas.SetLeft(number, 2);
                    Canvas.SetTop(number, Canvas.GetTop(backSeat));
                    TextBlock tbnumber = new TextBlock
                    {
                        Text = c.Number.ToString(),
                        FontSize = 8,
                        Margin = new Thickness(3, 0, 2, 0),
                    };
                    number.Child = tbnumber;
                    cv.Children.Add(number);
                } else {
                    // orientation == South

                    Canvas.SetLeft(desk, 0);
                    Canvas.SetTop(desk, 0);
                    cv.Children.Add(desk);

                    Canvas.SetLeft(backSeat, 23);
                    Canvas.SetTop(backSeat, 37);
                    cv.Children.Add(backSeat);

                    Canvas.SetLeft(frontSeat, 23);
                    Canvas.SetTop(frontSeat, 30);
                    cv.Children.Add(frontSeat);

                    Border number = new Border
                    {
                        CornerRadius = new CornerRadius(10),
                        BorderBrush = outlineBrush,
                        BorderThickness = new Thickness(0.8),
                    };
                    Canvas.SetLeft(number, 2);
                    Canvas.SetTop(number, Canvas.GetTop(frontSeat));
                    TextBlock tbnumber = new TextBlock
                    {
                        Text = c.Number.ToString(),
                        FontSize = 8,
                        Margin = new Thickness(3, 0, 2, 0),
                    };
                    number.Child = tbnumber;
                    cv.Children.Add(number);
                }

                // assume one person at least, with visa potentially empty => green => create a bool for emptyness and use it above
                // display cell number anyway

                if (isCellOccupied) {

                    Border visa = new Border
                    {
                        CornerRadius = new CornerRadius(3),
                        BorderBrush = visaBrush,
                        BorderThickness = new Thickness(0.8),
                    };
                    Canvas.SetLeft(visa, 7);
                    Canvas.SetTop(visa, Canvas.GetTop(desk) + 7);
                    TextBlock tbVisa = new TextBlock
                    {
                        Text = p.Visa,
                        Foreground = visaBrush,
                        FontSize = 8,
                        Margin = new Thickness(1, 0, 1, 0),
                    };
                    visa.Child = tbVisa;
                    cv.Children.Add(visa);
                }

                if (hasCellPhone) {

                    Border phone = new Border
                    {
                        CornerRadius = new CornerRadius(3),
                        BorderBrush = phoneBrush,
                        BorderThickness = new Thickness(0.8),
                    };
                    Canvas.SetLeft(phone, 29);
                    Canvas.SetTop(phone, Canvas.GetTop(desk) + 7);
                    TextBlock tbPhone = new TextBlock
                    {
                        Text = p.PhoneNumber,
                        Foreground = phoneBrush,
                        FontSize = 8,
                        Margin = new Thickness(1, 0, 1, 0),
                    };
                    phone.Child = tbPhone;
                    cv.Children.Add(phone);

                }


            }

            Content = cv;
        }
    }

    public class PointTypeConverterAndProvider : IValueConverter {

        internal static Dictionary<PointType, string> s_dictPointTypes;
        private static List<string> s_pointTypes;

        static PointTypeConverterAndProvider() {

            if (s_dictPointTypes == null) {

                s_dictPointTypes = new Dictionary<PointType, string>();

                s_dictPointTypes[PointType.WallStart] = "Wall Start";
                s_dictPointTypes[PointType.WallEnd] = "Wall End";
                s_dictPointTypes[PointType.WallJunction] = "Wall Junction";
                s_dictPointTypes[PointType.MovableWallStart] = "Thin Wall Start";
                s_dictPointTypes[PointType.MovableWallEnd] = "Thin Wall End";
                s_dictPointTypes[PointType.MovableWallJunction] = "Thin Wall Junction";
                s_dictPointTypes[PointType.DoorOpened] = "Door Inner Part";
                s_dictPointTypes[PointType.DoorPivot] = "Door Hinge";
                s_dictPointTypes[PointType.DoorClosed] = "Door End";
            }
            
        }

        public List<string> PointTypes {
            get { return s_pointTypes ?? (s_pointTypes = new List<string>(s_dictPointTypes.Values)); }
        }

        public object Convert(object value, Type targetType, object parameter, CultureInfo culture) {

            PointType pt = (PointType)value;
            if (! s_dictPointTypes.ContainsKey(pt)) {
                return 0;
            }

            return s_dictPointTypes[pt];
        }

        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture) {

            string val = (string)value;
            foreach (KeyValuePair<PointType, string> kvp in s_dictPointTypes) {
                if (kvp.Value == val) {
                    return (int) kvp.Key;
                }
            }

            return null;
        }
    }

    public class CustomTextBlock : ContentControl, IChangeChildren {

        public CustomTextBlock(string text) {
            Content = text;
            VerticalAlignment = VerticalAlignment.Center;
            HorizontalAlignment = HorizontalAlignment.Center;
        }

        public FrameworkElement Root {
            get { return this; }
        }

        public void Add(UIElement elt) {
        }

        public void RemoveById(string name) {
        }
    }

}
