﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.IO;
using System.Linq;
using System.ServiceModel.DomainServices.Client;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Input;
using RoomManagement.Views;
using RoomManagement.Web.Models;
using RoomManagement.Web.Services;

// If ChildWindows ever become modal like they should have been from the beginning, a lot of this'll have to be rewritten
// Currently it uses too much SilverMessageBox.Closed += delegate { ... }

namespace RoomManagement
{
    public partial class PeopleManagement : Page, INotifyPropertyChanged, IRefreshable, ICanBeBusy
    {
        private bool _mustLoadChanges = true;
        private bool _isLogShown;
        private readonly RoomContext _rc = new RoomContext();
        private readonly RoomContext _changesContext = new RoomContext(); // used to save only changes and not the rest

        private ObservableCollection<Person> _underlyingPeople;

        private PagedCollectionView _people;
        public PagedCollectionView People
        {
            get { return _people; }
            set
            {
                _people = value;
                FirePropertyChanged( "People" );
            }
        }
        private ObservableCollection<Person> _movedPeople;
        public ObservableCollection<Person> MovedPeople
        {
            get { return _movedPeople; }
            set
            {
                _movedPeople = value;
                FirePropertyChanged( "MovedPeople" );
            }
        }
        private ObservableCollection<Person> _addedPeople;
        public ObservableCollection<Person> AddedPeople
        {
            get { return _addedPeople; }
            set
            {
                _addedPeople = value;
                FirePropertyChanged( "AddedPeople" );
            }
        }
        private readonly ObservableCollection<Person> _modifiedPeople = new ObservableCollection<Person>();
        private IEnumerable<Site> _sites;
        public IEnumerable<Site> Sites
        {
            get { return _sites; }
            set
            {
                _sites = value;
                FirePropertyChanged( "Sites" );
            }
        }
        private IEnumerable<CellType> _cellTypes;
        public IEnumerable<CellType> CellTypes
        {
            get { return _cellTypes; }
            set
            {
                _cellTypes = value;
                FirePropertyChanged( "CellTypes" );
            }
        }
        public bool HasChanges { get { return ( _rc.HasChanges ); } }
        public bool IsBusy { get { return _rc.IsLoading || _rc.IsSubmitting; } }

        public event EventHandler IsBusyChanged;

        public PeopleManagement()
        {
            InitializeComponent();
            // do not set the DataContext now, as a timing problem seems to be at play:
            // sometimes, the bindings related to free cells computing are evaluated very early 
            // and people are not yet taken into account
            DataContext = null;

            if ( !WebContext.Current.User.IsAllocator )
            {
                DGpeople.IsReadOnly = true;
                MoveButton.IsEnabled = false;
                ChangesButton.IsEnabled = false;
                HelpImage1.Visibility = Visibility.Collapsed;
                HelpImage2.Visibility = Visibility.Collapsed;
            }
            if ( !WebContext.Current.User.IsCreator )
            {
                PersonActions.Visibility = Visibility.Collapsed;
            }
            if ( !WebContext.Current.User.IsAdministrator )
            {
                roomSelector.CanChangeCellType = false;
                _btnExport.Visibility = Visibility.Collapsed;
            }
            // 
            if (WebContext.Current.User.Name != @"ELCA\jmp") {
                DeletePersonButton.Visibility = Visibility.Collapsed;
            }

            MovedPeople = new ObservableCollection<Person>();
            AddedPeople = new ObservableCollection<Person>();
            _rc.PropertyChanged += OnContextPropertyChanged;

            Sites = _rc.Sites;
            CellTypes = _rc.CellTypes;
            LoadPeople( _rc.GetPeopleQuery(), true);
            LoadCellTypes( _rc.GetCellTypesQuery() ); // no need for it to be quick. used to set celltypes in MoveButton_Click
            LoadRoomTypes( _rc.GetRoomTypesQuery() ); // no need for it to be quick. used to see roomtypes in CellSelector
        }

        private void OnContextPropertyChanged( object sender, PropertyChangedEventArgs e )
        {
            if ( e.PropertyName == "HasChanges" || e.PropertyName == "People" )
            {
                FirePropertyChanged( e.PropertyName );
            }
            if ( e.PropertyName == "IsLoading" || e.PropertyName == "IsSubmitting" )
            {
                if ( IsBusyChanged != null )
                {
                    IsBusyChanged( this, EventArgs.Empty );
                }
            }
        }

        private void SearchButton_Click( object sender, RoutedEventArgs e )
        {
            List<Site> SelectedSites = SitesSelector.SelectedSites;
            SitesSelector.SelectedSites = new List<Site>();
            if ( string.IsNullOrWhiteSpace( TBfirstNameSearch.Text ) && string.IsNullOrWhiteSpace( TBlastNameSearch.Text ) &&
                 string.IsNullOrWhiteSpace( TBvisaSearch.Text ) && SelectedSites.Count == 0 ) return;

            if ( !string.IsNullOrWhiteSpace( TBvisaSearch.Text ) )
            {
                LoadPeople( _rc.SearchPeopleByVisaQuery( TBvisaSearch.Text ), false );
            }
            else if ( !string.IsNullOrWhiteSpace( TBfirstNameSearch.Text ) || !string.IsNullOrWhiteSpace( TBlastNameSearch.Text ) )
            {
                LoadPeople(_rc.SearchPeopleQuery(TBfirstNameSearch.Text, TBlastNameSearch.Text), false);
            }
            else
            {
                LoadPeople(_rc.GetPeopleFromSitesQuery(from s in SelectedSites select s.Location), false);
            }
            ShowAllButton.IsEnabled = true;
        }
        private void ShowAllButton_Click( object sender, RoutedEventArgs e )
        {
            LoadPeople(_rc.GetPeopleQuery(), false);
            SitesSelector.ClearText();
            TBvisaSearch.Text = string.Empty;
            TBfirstNameSearch.Text = string.Empty;
            TBlastNameSearch.Text = string.Empty;
            ShowAllButton.IsEnabled = false;
        }
        private void SaveButton_Click( object sender, RoutedEventArgs e )
        {
            if ( _modifiedPeople.Count( p => p.HasValidationErrors ) > 0 ) return;
            if (!DGpeople.IsValid) return;

            Dictionary<string, int> workplaces = new Dictionary<string, int>();
            foreach ( Person p in MovedPeople.Where( p => !string.IsNullOrWhiteSpace( p.NewWorkPlace ) ) )
            {
                if ( !workplaces.Keys.Contains( p.NewWorkPlace ) )
                {
                    workplaces.Add( p.NewWorkPlace, 1 );
                }
                else
                {
                    workplaces[p.NewWorkPlace]++;
                }
            }
            foreach ( Person p in People )
            {
                if ( p.WorkPlace != null && workplaces.Keys.Contains( p.WorkPlace ) )
                {
                    workplaces[p.WorkPlace]++;
                }
            }
            IEnumerable<string> newWorkPlaces = from p in MovedPeople select p.NewWorkPlace;

            // A non-desksharing cell can't have more than one person
            foreach ( Person p in People )
            {
                if ( p.Cell != null && p.Cell.CellTypeID != Constants.DeskSharingCellTypeId && newWorkPlaces.Contains( p.WorkPlace ) )
                {
                    SilverMessageBox.Show( "Someone is already in the cell " + p.WorkPlace );
                    return;
                }
            }

            // NewWorkPlaces must be valid
            InvokeOperation io = _rc.AreWorkPlacesOK( workplaces );
            io.Completed += delegate
            {
                string result = (string) io.Value;
                if ( result == Constants.AreNewWorkplacesOKResultWhenOK ) // OK
                {
                    Save(); // forced to put that here...else it's done before io is completed
                    MovedPeople.Clear();
                }
                else if ( result == null ) // At least one is not valid
                {
                    foreach ( string workPlace in newWorkPlaces )
                    {
                        string tempWorkPlace = workPlace;
                        io = _rc.IsWorkPlaceOK( workPlace );
                        io.Completed += delegate
                        {
                            SilverMessageBox.Show( "The cell " + tempWorkPlace + " is in an incorrect format or does not exist." );
                        };
                    }
                }
                else // At least one has at least 2 people
                {
                    SilverMessageBox.Show( "Error: There is more than one person in " + result + ", which is not a desk sharing cell and thus cannot contain more than one person." );
                }
            };
        }
        private void RejectButton_Click( object sender, RoutedEventArgs e ) {

            DGpeople.CommitEdit();

            SilverMessageBox smb = SilverMessageBox.Show( "Do you really want to reject all the changes you just made ?", "Yes", button2Text: "No" );
            smb.Closed += delegate
            {
                if ( smb.DialogResult == true ) // Yes
                {
                    Reject();
                    ShowAllButton.IsEnabled = false;
                }
            };
            MovedPeople.Clear();
        }
        private void MoveButton_Click( object sender, RoutedEventArgs e ) {

            Cell c = roomSelector.SelectedCell;

            if (c == null) {
                SilverMessageBox.Show("Please select a cell.");
                return;
            }

            Person p = (Person) DGpeople.SelectedItem;
            if ( p == null )
            {
                SilverMessageBox.Show( "Please select someone to move." );
                return;
            }

            if ( !c.IsEquipped && c.CellTypeID != Constants.DeskSharingCellTypeId )
            {
                SilverMessageBox.Show( "This cell is not equipped ; no one can work in it." );
                return;
            }
            if ( c.CellTypeID == Constants.VisitorCellTypeId )
            {
                SilverMessageBox.Show( "This is a visitor cell ; no one can be assigned to it." );
                return;
            }
            if ( WebContext.Current.User.IsAdministrator && c.CellTypeID != Constants.DeskSharingCellTypeId && c.People != null && c.People.Count == 1 )
            {
                SilverMessageBox smb = SilverMessageBox.Show( "This cell is not a desk sharing cell and thus cannot contain more than one person.\nWould you like to change it into a desksharing cell and move the selected person in it?", button1Text: "Yes", button2Text: "No" );
                smb.Closed += delegate
                {
                    if ( smb.DialogResult == true ) // Yes
                    {
                        c.CellType = _rc.CellTypes.Where( ct => ct.ID == Constants.DeskSharingCellTypeId ).First();
                        p.ChangeCell( c );
                        AddSelectedItemToMovedPeople();
                    }
                };
            }
            else if ( c.CellTypeID != Constants.DeskSharingCellTypeId && c.People != null && c.People.Count == 1 )
            {
                SilverMessageBox.Show( "This cell is not a desk sharing cell and thus cannot contain more than one person." );
                return;
            }
            else
            {
                p.ChangeCell( c );
                AddSelectedItemToMovedPeople();
            }
            DGpeople.BeginEdit();
        }
        private void ChangesButton_Click( object sender, RoutedEventArgs e )
        {
            if ( _isLogShown )
            {
                LogRow.Height = new GridLength( 0 );
                ChangesButton.Content = "View changes";
            }
            else
            {
                LogRow.Height = new GridLength( 1, GridUnitType.Star );
                ChangesButton.Content = "Hide changes";
            }
            _isLogShown = !_isLogShown;
        }
        private void AddPersonButton_Click( object sender, RoutedEventArgs e ) {
            DGpeople.CommitEdit();

            AddPersonWindow apw = AddPersonWindow.ShowDialog();
            apw.Closed += delegate
            {
                if ( apw.DialogResult == true ) // OK
                {
                    Person p = apw.AddedPerson;

                    Dispatcher.BeginInvoke(delegate {
                                               
                        _underlyingPeople.Add(p);
                        _rc.Persons.Add( p);
                        DGpeople.ScrollIntoView(p, DGpeople.Columns[0]);
                        // If there are validation errors, show them
                        DGpeople.SelectedItem = p;
                        DGpeople.BeginEdit();
                    });
                }
            };
        }

        private void DeletePersonButton_Click( object sender, RoutedEventArgs e ) {

            DGpeople.CommitEdit();
            SilverMessageBox smb = SilverMessageBox.Show( "Do you really want to delete this worker?", button1Text: "Yes", button2Text: "No" );
            Person p = (Person) DGpeople.SelectedItem;
            smb.Closed += delegate
            {
                if ( smb.DialogResult == true ) // Yes
                {
                    People.Remove( p );
                    _rc.Persons.Remove( p );
                }
            };
        }

        private void GenerateHTMLButton_Click( object sender, RoutedEventArgs e )
        {
            List<Person> addedAndMovedPeople = MovedPeople.Intersect( AddedPeople ).ToList();

            if ( MovedPeople.Count == 0 && AddedPeople.Count == 0 )
            {
                SilverMessageBox.Show( "No one was moved or added." );
                return;
            }
            string date = TBdate.Text;
            if ( string.IsNullOrWhiteSpace( date ) )
            {
                SilverMessageBox.Show( "Please specify a date." );
                return;
            }
            StringBuilder html = new StringBuilder();
            if ( AddedPeople.Count > 0 )
            {
                html.Append( "New People: <br>" );
                html.Append( "<table border=\"1\" width=800px style=\"background:#f0f0f0\"><tbody>" );
                html.Append( "<tr>" );
                html.Append( "<th>Visa</th>" );
                html.Append( "<th>First Name</th>" );
                html.Append( "<th>Last Name</th>" );
                html.Append( "<th>Workplace</th>" );
                html.Append( "<th>Phone number</th>" );
                html.Append( "<th>Date</th>" );
                html.Append( "</tr>" );
                foreach ( Person p in AddedPeople )
                {
                    html.Append( "<tr>" );
                    html.Append( "<td>" + p.Visa + "</td>" );
                    html.Append( "<td>" + p.FirstName + "</td>" );
                    html.Append( "<td>" + p.LastName + "</td>" );
                    html.Append( "<td>" + ( string.IsNullOrWhiteSpace( p.WorkPlace ) ? "none" : p.WorkPlace ) + "</td>" );
                    html.Append( "<td>" + ( string.IsNullOrWhiteSpace( p.PhoneNumber ) ? "none" : p.PhoneNumber ) + "</td>" );
                    html.Append( "<td>" + date + "</td>" );
                    html.Append( "</tr>" );
                }
                html.Append( "</tbody></table>" );
            }
            List<Person> movedPeople = MovedPeople.Except( addedAndMovedPeople ).ToList();
            if ( movedPeople.Count() > 0 )
            {
                html.Append( "Moved People: <br>" );
                html.Append( "<table border=\"1\" width=800px style=\"background:#f0f0f0\"><tbody>" );
                html.Append( "<tr>" );
                html.Append( "<th>Visa</th>" );
                html.Append( "<th>First Name</th>" );
                html.Append( "<th>Last Name</th>" );
                html.Append( "<th>Old Workplace</th>" );
                html.Append( "<th>New Workplace</th>" );
                html.Append( "<th>Old Phone Number</th>" );
                html.Append( "<th>New Phone Number</th>" );
                html.Append( "<th>Date</th>" );
                html.Append( "</tr>" );
                foreach ( Person p in movedPeople )
                {
                    html.Append( "<tr>" );
                    html.Append( "<td>" + p.Visa + "</td>" );
                    html.Append( "<td>" + p.FirstName + "</td>" );
                    html.Append( "<td>" + p.LastName + "</td>" );
                    html.Append( "<td>" + ( string.IsNullOrWhiteSpace( p.WorkPlace ) ? "none" : p.WorkPlace ) + "</td>" );
                    html.Append( "<td>" + p.NewWorkPlace + "</td>" );
                    html.Append( "<td>" + ( string.IsNullOrWhiteSpace( p.OldPhoneNumber ) ? "none" : p.OldPhoneNumber ) + "</td>" );
                    html.Append( "<td>" + ( p.HasPhoneNumberChanged ? "did not change" : p.PhoneNumber ) + "</td>" );
                    html.Append( "<td>" + date + "</td>" );
                    html.Append( "</tr>" );
                }
                html.Append( "</tbody></table> <br>" );
            }
            string table = html.ToString();
            Clipboard.SetText( table );

            SilverMessageBox.Show( "The HTML code has been copied into your clipboard." );
        }
        private void UndoButton_Click( object sender, RoutedEventArgs e )
        {
            Person p = (Person) ( (Button) sender ).DataContext;
            p.PhoneNumber = p.OldPhoneNumber;
            p.NewWorkPlace = null;
            MovedPeople.Remove( p );
        }
        private void SaveForLaterButton_Click( object sender, RoutedEventArgs e )
        {
            int id = 0;
            foreach ( Person p in MovedPeople )
            {
                Change c = new Change {ID = id++, PersonID = p.ID};
                if ( p.HasPhoneNumberChanged )
                {
                    c.NewPhoneNumber = p.PhoneNumber;
                }
                c.NewWorkPlace = p.NewWorkPlace;
                c.Author = WebContext.Current.User.Name;
                c.Flags = p.Flags;
                _changesContext.Changes.Add( c );
            }
            foreach (Person p in _modifiedPeople) {
                if (! MovedPeople.Contains(p)) {
                    
                    Change c = new Change {ID = id++, PersonID = p.ID};
                    if ( p.HasPhoneNumberChanged )
                    {
                        c.NewPhoneNumber = p.PhoneNumber;
                    }
                    c.NewWorkPlace = null; // otherwise would be in MovedPeople
                    c.Author = WebContext.Current.User.Name;
                    c.Flags = p.Flags;
                    _changesContext.Changes.Add( c );
                }
            }
            SaveForLaterButton.IsEnabled = false;
            _changesContext.SubmitChanges( delegate { SaveForLaterButton.IsEnabled = true; }, null );
        }

        private void SearchTBs_KeyDown( object sender, KeyEventArgs e )
        {
            if ( e.Key == Key.Enter )
            {
                SearchButton_Click( null, null ); // HACK to replace IsDefault
            }
        }

        private void DGpeople_CellEditEnded( object sender, DataGridCellEditEndedEventArgs e )
        {
            Person p = (Person) DGpeople.SelectedItem;
            if ( e.EditAction == DataGridEditAction.Commit && e.Column.Header.ToString() == "New Work Place" )
            {
                if ( !MovedPeople.Contains( p ) )
                {
                    AddSelectedItemToMovedPeople();
                }
                // If the person's site has changed, add a phone number warning
                string siteName = p.NewWorkPlace.Split( ' ' )[0];
                foreach ( Site s in Sites )
                {
                    if ( siteName == s.Name && p.Cell != null && p.Cell.Room.Floor.Site != s && !p.HasPhoneNumberChanged )
                    {
                        p.PhoneNumber += '!'; // to force PhoneNumber change
                        DGpeople.BeginEdit();
                    }
                }
            }
            if ( !_modifiedPeople.Contains( p ) )
            {
                _modifiedPeople.Add( p );
            }
            p.RaisePhoneAndWorkPlaceChanges(); // else, the MovedPeople datagrid won't be updated if that person is already in it
        }

        private void RefreshPeople( SubmitOperation so )
        {
            LoadPeople( _rc.GetPeopleBypassingCacheQuery(), false );
        }

        /// <summary>
        /// Refreshes everything
        /// </summary>
        public void RefreshData()
        {
            _rc.RefreshSettingsData();
            LoadPeople( _rc.GetPeopleBypassingCacheQuery(), true );
        }

        private void LoadPeople( EntityQuery<Person> query, bool alsoLoadRoomsData)
        {
            _rc.Load( query, LoadBehavior.MergeIntoCurrent, OnLoadedPeopleData, alsoLoadRoomsData );
        }
        private void LoadSites( EntityQuery<Site> query )
        {
            _rc.Sites.Clear();
            _rc.Load( query, LoadBehavior.RefreshCurrent, OnLoadedSitesData, null );
        }
        private void LoadFloors( EntityQuery<Floor> query )
        {
            _rc.Floors.Clear();
            _rc.Load( query, LoadBehavior.RefreshCurrent, OnLoadedFloorsData, null );
        }
        private void LoadCellTypes( EntityQuery<CellType> query )
        {
            _rc.CellTypes.Clear();
            _rc.Load( query, LoadBehavior.RefreshCurrent, OnLoadedTypesData, null );
        }
        private void LoadRoomTypes( EntityQuery<RoomType> query )
        {
            _rc.RoomTypes.Clear();
            _rc.Load( query, LoadBehavior.RefreshCurrent, OnLoadedTypesData, null );
        }
        private void OnLoadedPeopleData( LoadOperation e )
        {
            if ( e.HasError )
            {
                SilverMessageBox.Show( Constants.ErrorMessage, "OK", e.Error.ToString() );
                e.MarkErrorAsHandled();
                return;
            }

            _underlyingPeople = new ObservableCollection<Person>();
            foreach ( Entity ent in e.Entities )
            {
                _underlyingPeople.Add( (Person) ent );
            }
            People = new PagedCollectionView(_underlyingPeople);

            bool alsoLoadRoomsData = (bool) e.UserState;

            if (alsoLoadRoomsData) {
                // HACK to make sure Floors are loaded before Sites (else, all sites have no floors in the beginning => they appear to have 0 free)
                // and both must load AFTER People, else all Cells appear to be free
                LoadFloors(_rc.GetFloorsQuery());
            }

            #region Load Saved Changes
            if ( _mustLoadChanges )
            {
                LoadOperation<Change> lo = _changesContext.Load( _changesContext.GetChangesQuery() );
                lo.Completed += delegate
                {
                    IEnumerable<Change> changes = lo.Entities;
                    if ( changes.Count() > 0 )
                    {
                        SilverMessageBox.Show( "Changes previously made by " + changes.ElementAt( 0 ).Author + " were automatically loaded." );
                    }
                    foreach ( Change c in changes )
                    {
                        Guid PersonID = c.PersonID;
                        Person changedPerson = _underlyingPeople.Where( p => p.ID == PersonID ).First();
                        if ( !string.IsNullOrWhiteSpace( c.NewPhoneNumber ) )
                        {
                            changedPerson.PhoneNumber = c.NewPhoneNumber;
                            _modifiedPeople.Add( changedPerson );
                        }

                        changedPerson.Flags = c.Flags;
                        changedPerson.NewWorkPlace = c.NewWorkPlace;
                        MovedPeople.Add( changedPerson );
                    }
                    _changesContext.DeleteAllChanges();
                };
                _mustLoadChanges = false;
            }
            #endregion
        }
        private void OnLoadedSitesData( LoadOperation e )
        {
            if ( e.HasError )
            {
                SilverMessageBox.Show( Constants.ErrorMessage, "OK", e.Error.ToString() );
                e.MarkErrorAsHandled();
                return;
            }

            if ( DataContext == null )
            {
                DataContext = this;
            }

            foreach (Cell c in _rc.Cells) {
                c.RaiseAllChanges();
            }
        }
        private void OnLoadedFloorsData( LoadOperation e )
        {
            if ( e.HasError )
            {
                SilverMessageBox.Show( Constants.ErrorMessage, "OK", e.Error.ToString() );
                e.MarkErrorAsHandled();
                return;
            }
            LoadSites( _rc.GetSitesQuery() );
        }
        private void OnLoadedTypesData( LoadOperation e )
        {
            if ( e.HasError )
            {
                SilverMessageBox.Show( Constants.ErrorMessage, "OK", e.Error.ToString() );
                e.MarkErrorAsHandled();
                return;
            }
        }

        private void Save()
        {
            DGpeople.CommitEdit( DataGridEditingUnit.Row, true );
            _rc.SubmitChanges( RefreshPeople, null );
            _changesContext.DeleteAllChanges();
        }
        private void Reject() {

            _rc.RejectChanges();

            LoadPeople( _rc.GetPeopleQuery(), true );
            _changesContext.DeleteAllChanges();
        }

        private void AddSelectedItemToMovedPeople()
        {
            Person p = (Person) DGpeople.SelectedItem;
            if ( String.IsNullOrWhiteSpace( p.NewWorkPlace ) && MovedPeople.Contains( p ) )
            {
                MovedPeople.Remove( p );
            }
            else if ( !MovedPeople.Contains( p ) )
            {
                MovedPeople.Add( p );
            }
        }

        private void ExportToFileButton_Click( object sender, RoutedEventArgs e )
        {
            SaveFileDialog dialog = new SaveFileDialog { DefaultExt = "csv", Filter = "CSV Files (*.csv)|*.csv", FilterIndex = 1 };
            if ( dialog.ShowDialog() == true )
            {
                using ( StreamWriter writer = new StreamWriter( dialog.OpenFile(), Encoding.UTF8 ) )
                {
                    writer.Write( CreateFile() );
                }
            }
        }
        private void ExportToClipboardButton_Click( object sender, RoutedEventArgs e )
        {
            Clipboard.SetText( CreateFile( end: "\"\t" ) ); // Excel likes tabs as separator
            SilverMessageBox.Show( "The grid's content has been successfully copied to the Clipboard." );
        }

        private void ExportToRpisButton_Click(object sender, RoutedEventArgs e) {
            _rc.GenerateRpisFiles(delegate (InvokeOperation io) {
                if (io.HasError) {
                    
                    SilverMessageBox.Show(io.Error.Message);
                    io.MarkErrorAsHandled();
                } else {
                    SilverMessageBox.Show("The RPIS files have been generated.");
                }
                                  }, null);
        }

        private string CreateFile( string begin = "\"", string end = "\";" )
        {   // Default = CSV
            StringBuilder file = new StringBuilder();

            file.Append( begin + "First Name" + end );
            file.Append( begin + "Last Name" + end );
            file.Append( begin + "Visa" + end );
            file.Append( begin + "Phone Number" + end );
            file.Append( begin + "Email Address" + end );
            file.Append( begin + "Work Place" + end );
            file.Append( Environment.NewLine );
            foreach ( Person p in People )
            {
                file.Append( begin + p.FirstName + end );
                file.Append( begin + p.LastName + end );
                file.Append( begin + p.Visa + end );
                file.Append( begin + p.PhoneNumber + end );
                file.Append( begin + p.Email + end );
                file.Append( begin + p.WorkPlace + end );
                file.Append( Environment.NewLine );
            }
            return file.ToString();
        }

        #region INotifyPropertyChanged
        public event PropertyChangedEventHandler PropertyChanged;
        private void FirePropertyChanged( string propertyName )
        {
            if ( PropertyChanged != null )
            {
                PropertyChanged( this, new PropertyChangedEventArgs( propertyName ) );
            }
        }
        #endregion
    }
}